# -*- coding: utf-8  -*-
"""Match package."""

# Copyright (C) 2015 Alexander Jones
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import pkg_resources


class MatchConfig(object):

    """Match configuration singleton class."""

    created = False

    def __init__(self):
        """Constructor."""
        if MatchConfig.created:
            raise RuntimeError
        MatchConfig.created = True
        self._base_matches = {}
        for match in pkg_resources.iter_entry_points(group='competitions.match.base'):
            self._base_matches.update({match.name: match.load()})
        self.base_match = 'competitions.simple'

    @property
    def base_match(self):
        """The "base" match class."""
        return self._base_matches.get(self._base_match, None)

    @base_match.setter
    def base_match(self, match_def):
        """Set the "base" match class.

        @param match_def: The code of the new "base" match class
        @type match_def: str
        """
        self._base_match = match_def


config = MatchConfig()



# -*- coding: utf-8  -*-
"""Base class for matches."""

# Copyright (C) 2015 Alexander Jones
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals


class Match(object):

    """Base class for matches."""

    def play(self):
        """Play the match."""
        raise NotImplementedError


class TwoCompetitorMatch(Match):

    """Base class for matches with exactly two competitors."""

    @property
    def competitor1(self):
        """The first (usually home) competitor."""
        raise NotImplementedError

    @property
    def competitor2(self):
        """The second (usually away) competitor."""
        raise NotImplementedError

    @property
    def is_walkover(self):
        """Whether or not this match is a walkover/bye."""
        return self.competitor1 is None or self.competitor2 is None


class TwoTeamMatch(TwoCompetitorMatch):

    """Base class for two-team matches."""

    def __init__(self, team1, team2):
        """Constructor.

        @param team1: The first (home) team
        @type team1: An object that can be converted to a string
        @param team2: The second (away) team
        @type team2: An object that can be converted to a string
        """
        self.team1 = team1
        self.team2 = team2

    @property
    def competitor1(self):
        """The first (usually home) competitor."""
        raise self.team1

    @property
    def competitor2(self):
        """The second (usually away) competitor."""
        raise self.team2


class TwoPlayerMatch(TwoCompetitorMatch):

    """Base class for two-player matches."""

    def __init__(self, player1, player2):
        """Constructor.

        @param player1: The first (home) player
        @type player1: An object that can be converted to a string
        @param player2: The second (away) player
        @type player2: An object that can be converted to a string
        """
        self.player1 = player1
        self.player2 = player2

    @property
    def competitor1(self):
        """The first (usually home) competitor."""
        raise self.player1

    @property
    def competitor2(self):
        """The second (usually away) competitor."""
        raise self.player2



# -*- coding: utf-8  -*-
"""Tests for match class registration and use."""

# Copyright (C) 2015 Alexander Jones
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from . import TestCase

from competitions.match import MatchConfig, config
from competitions.match.default.SimpleMatch import SimpleMatch
from competitions.match.default.TestMatch import TestMatch


class TestMatchRegistration(TestCase):

    """Tests for match class finding."""

    def test_basic(self):
        """Test default and explicit match classes."""
        self.assertEqual(config.base_match, SimpleMatch)
        config.base_match = 'competitions.test'
        self.assertEqual(config.base_match, TestMatch)
        config.base_match = 'competitions.unused'
        self.assertIsNone(config.base_match)

    def test_singleton(self):
        """Test to ensure the configuration object's singleton status."""
        self.assertRaises(RuntimeError, MatchConfig)


class TestSimpleMatch(TestCase):

    """Basic sanity checks for SimpleMatch."""

    def test_simulation(self):
        """Test SimpleMatch construction and simulation."""
        config.base_match = 'competitions.simple'
        team1 = 'First'
        team2 = 'Second'
        for __ in range(1000):
            match = config.base_match(team1, team2)
            self.assertIsInstance(match, SimpleMatch, 'Wrong class.')
            self.assertEqual(match.team1, team1)
            self.assertEqual(match.team2, team2)
            match.play()
            if match.score1 > match.score2:
                self.assertEqual(match.winner, team1)
            elif match.score2 > match.score1:
                self.assertEqual(match.winner, team2)
            else:
                self.assertIsNone(match.winner)
            shortstr = '{}-{}'.format(match.score1, match.score2)
            self.assertEqual(match.score_str(), shortstr)
            longstr = '{} {} - {} {}'.format(team1, match.score1, match.score2,
                                             team2)
            self.assertEqual(str(match), longstr)


class TestTestMatch(TestCase):

    """Basic sanity checks for TestMatch."""

    def test_simulation(self):
        """Test TestMatch construction and simulation."""
        config.base_match = 'competitions.test'
        team1 = 'First'
        team2 = 'Second'
        for __ in range(1000):
            match = config.base_match(team1, team2)
            self.assertIsInstance(match, TestMatch, 'Wrong class.')
            self.assertEqual(match.team1, team1)
            self.assertEqual(match.team2, team2)
            match.play()
            self.assertEqual(match.score1, 5)
            self.assertEqual(match.score2, 0)
            self.assertEqual(match.winner, team1)
            shortstr = '5-0'
            self.assertEqual(match.score_str(), shortstr)
            longstr = 'First 5 - 0 Second'
            self.assertEqual(str(match), longstr)



# -*- coding: utf-8  -*-
"""Match simulator for tests."""

# Copyright (C) 2015 Alexander Jones
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

from competitions.match.Match import TwoTeamMatch


class TestMatch(TwoTeamMatch):

    """A match simulator for tests."""

    def __init__(self, team1, team2):
        """Constructor.

        @param team1: The first (home) team
        @type team1: An object that can be converted to a string
        @param team2: The second (away) team
        @type team2: An object that can be converted to a string
        """
        super(TestMatch, self).__init__(team1, team2)
        self.score1 = ''
        """The score for team1."""
        self.score2 = ''
        """The score for team2."""

    def __unicode__(self):
        """Return a unicode representation of the match.

        This will be of the form "<team1> <score1> - <score2> <team2>".
        @return: The match as a unicode string
        @rtype: unicode
        """
        return '{0} {1} - {2} {3}'.format(self.team1, self.score1,
                                          self.score2, self.team2)

    def __str__(self):
        """Return a str representation of the match.

        The format of the string is the same as that returned by __unicode__,
        except that it is a str object instead of a unicode object.
        @return: The match as a str
        @rtype: str
        """
        return str(self.__unicode__())

    def score_str(self):
        """Return the score of the match as a string.

        @return: The score of the match.
        @rtype: str
        """
        return '{0}-{1}'.format(self.score1, self.score2)

    def play(self):
        """Play the match.

        This simulator will always result in a 5-0 win for team1.
        @return: The winner (or None if the result is a draw)
        @rtype: An object that can be converted to a string or NoneType
        """
        self.score1 = 5
        self.score2 = 0
        self.winner = self.team1
        self.loser = self.team2
        return self.winner



# -*- coding: utf-8  -*-
"""Simple default match simulator."""

# Copyright (C) 2015 Alexander Jones
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from __future__ import unicode_literals

import random

from competitions.match.Match import TwoTeamMatch


class SimpleMatch(TwoTeamMatch):

    """A simple default match simulator based on the card game "War"."""

    def __init__(self, team1, team2):
        """Constructor.

        @param team1: The first (home) team
        @type team1: An object that can be converted to a string
        @param team2: The second (away) team
        @type team2: An object that can be converted to a string
        """
        super(SimpleMatch, self).__init__(team1, team2)
        self.score1 = ''
        """The score for team1."""
        self.score2 = ''
        """The score for team2."""

    def __unicode__(self):
        """Return a unicode representation of the match.

        This will be of the form "<team1> <score1> - <score2> <team2>".
        @return: The match as a unicode string
        @rtype: unicode
        """
        return '{0} {1} - {2} {3}'.format(self.team1, self.score1,
                                          self.score2, self.team2)

    def __str__(self):
        """Return a str representation of the match.

        The format of the string is the same as that returned by __unicode__,
        except that it is a str object instead of a unicode object.
        @return: The match as a str
        @rtype: str
        """
        return str(self.__unicode__())

    def score_str(self):
        """Return the score of the match as a string.

        @return: The score of the match.
        @rtype: str
        """
        return '{0}-{1}'.format(self.score1, self.score2)

    def play(self):
        """Play the match.

        This match simulator iterates through two lists of random numbers
        25 times, one for each team, comparing the numbers and awarding a point
        to the team with the higher number. The team with more points at the
        end of the lists wins and is recorded in the winner field. If the result
        is a draw, the winner field is set to None.

        @return: The winner (or None if the result is a draw)
        @rtype: An object that can be converted to a string or NoneType
        """
        score1 = 0
        score2 = 0
        for __ in range(25):
            num1 = random.randint(0, 100)
            num2 = random.randint(0, 100)
            if num1 > num2:
                score1 += 1
            elif num2 > num1:
                score2 += 1
        if score1 > score2:
            self.winner = self.team1
            self.loser = self.team2
        elif score2 > score1:
            self.winner = self.team2
            self.loser = self.team1
        else:
            self.winner = None
            self.loser = None
        self.score1 = score1
        self.score2 = score2
        return self.winner



# -*- coding: utf-8  -*-
"""A setuptools based setup module."""

from setuptools import setup, find_packages

from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='competitions-match',
    version='0.2',

    description='competitions support library for matches',
    long_description=long_description,

    url='https://github.com/happy5214/competitions-match',

    author='Alexander Jones',
    author_email='happy5214@gmail.com',

    license='LGPLv3+',
    classifiers=[
        'Development Status :: 3 - Alpha',

        'License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)',

        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    keywords='competitions matches',

    packages=find_packages(exclude=['docs', 'tests*']),

    namespace_packages=['competitions'],

    test_suite='tests',

    entry_points={
        'competitions.match.base': [
            'competitions.simple = competitions.match.default.SimpleMatch:SimpleMatch',
            'competitions.test = competitions.match.default.TestMatch:TestMatch'
        ]
    },
)
